/******************************************************************************
  Filename:        cc112x_dsss_mode_and_t_mode.c
  
  Description: 
  
Notes:          DSSS packet:

                Preamble: 0101010101010101010101 (not DSSS coded -> 22 chips)
                Header:   0xAAA (DSSS coded -> 132 chips)
                Sync:     0xDE (DSSS coded -> 88 chips)
                Payload:  15 bytes (DSSS coded -> 1320 chips)
                CRC:      2 bytes (DSSS coded -> 176 chips)
                
                see  https://de.wikipedia.org/wiki/Barker-Code
                +1 +1 +1 -1 -1 -1 +1 -1 -1 +1 -1 = 0x712
                1 : 111.0001.0010 (0x0712) 
                0 : 000.1110.1101 (0x00ED) 

                T-Mode packet: regular wM-Bus packet in Frame Format A
                C-mode packet: regular wM-Bus packet in Frame Format B

                Code example is always in RX looking for valid T-mode/C-mode packets or DSSS ones
******************************************************************************/


/*****************************************************************************
* INCLUDES
*/
#include  <msp430.h>
#include "hal_board.h"
#include "hal_mcu.h"
#include "hal_led_trxeb.h"
#include "hal_button_trxeb.h"
#include "lcd_dogm128_6.h"   
#include "hal_spi_rf_trxeb.h"
#include "hal_int_rf_trxeb.h"
#include "cc112x_spi.h"
#include "cc112x_dsss_mode_and_t_mode_reg_config.h"
#include "crc.h"
#include "3outof6.h"
#include "mbus_packet.h"
/******************************************************************************
* LOCAL VARIABLES
*/

// Struct. used to hold information used for RX
typedef struct RXinfoDescr {
    uint16 lengthField;         // The L-field in the WMBUS packet
    uint16 length;              // Total number of bytes to to be read from the RX FIFO
    uint16 bytesLeft;           // Bytes left to to be read from the RX FIFO
    uint8 *pByteIndex;          // Pointer to current position in the byte array
    uint8 format;               // Infinite or fixed packet mode
    uint8 start;                // Start of Packet
    uint8 complete;             // Packet received complete
    uint8 mode;                 // S-mode or T-mode
} RXinfoDescr;

RXinfoDescr RXinfo;  // T-mode RX  & S-mode RX

#define INFINITE                  0
#define FIXED                     1

static uint16 timeout = 4000;           // Timeout for 4 bytes sync searc after 
                                        // DSSS sync word (2 byte) is found. 
                                        // This timeout should be made as small 
                                        // as possible
static uint8 syncReceived = FALSE;      // Flag asserted on rising edge 
                                        // of GPIO3 (PKT_SYNC_RXTX)


static uint32 packetCounterDSSSMode = 0;
static uint32 errorCounterDSSSMode = 0;

static uint32 packetCounterTMode = 0;
static uint32 errorCounterTMode = 0;
static uint32 errorCounterCRCTMode = 0;

static uint32 packetCounterCMode = 0;
static uint32 errorCounterCMode = 0;
static uint32 errorCounterCRCCMode = 0;

uint8 writeByte;
uint8 readByte;
uint8 bytesDecoded[2];
uint16 rxStatus = 0;


/******************************************************************************
* STATIC FUNCTIONS
*/
static void initMCU(void);
static void registerConfig(void);
static void manualCalibration(void);
static void updateLcd(void);
static void packetReceivedISR(void);
static void syncReceivedISR(void);
static void fifoThresholdReachedISR(void);
static void partition(uint8* dataIn, uint8 n, uint8 nBytes);
static void recreatePacket(void);


/******************************************************************************
* DEFINES
*/
#define SYNC_RECEIVED           GPIO_2
#define FIFO_THRESHOLD_REACHED  GPIO_0
#define PACKET_RECEIVED         GPIO_3

//0xAF12 = 5 last bits from 0x55 preamble + 11 bits from 0x712
#define DUAL_SYNC_SETTINGS() \
do { \
  uint8 writeByte; \
  writeByte = 0x08; \
  cc112xSpiWriteReg(CC112X_SYNC_CFG1, &writeByte, 1); \
  writeByte = 0x1F; \
  cc112xSpiWriteReg(CC112X_SYNC_CFG0, &writeByte, 1); \
  writeByte = 0x54; \
  cc112xSpiWriteReg(CC112X_SYNC3, &writeByte, 1); \
  writeByte = 0x3D; \
  cc112xSpiWriteReg(CC112X_SYNC2, &writeByte, 1); \
  writeByte = 0xAF; \
  cc112xSpiWriteReg(CC112X_SYNC1, &writeByte, 1); \
  writeByte = 0x12; \
  cc112xSpiWriteReg(CC112X_SYNC0, &writeByte, 1); \
} while (0)

//0x3B789712 = last 10 chips of "0' (last bit in 0xAAA) + 22 chips from first 2 Bits of 0xDE or "11"
#define _4_BYTES_SYNC_SETTINGS() \
do { \
  uint8 writeByte; \
  writeByte = 0x0A; \
  cc112xSpiWriteReg(CC112X_SYNC_CFG1, &writeByte, 1); \
  writeByte = 0x17; \
  cc112xSpiWriteReg(CC112X_SYNC_CFG0, &writeByte, 1); \
  writeByte = 0x3B; \
  cc112xSpiWriteReg(CC112X_SYNC3, &writeByte, 1); \
  writeByte = 0x78; \
  cc112xSpiWriteReg(CC112X_SYNC2, &writeByte, 1); \
  writeByte = 0x97; \
  cc112xSpiWriteReg(CC112X_SYNC1, &writeByte, 1); \
  writeByte = 0x12; \
  cc112xSpiWriteReg(CC112X_SYNC0, &writeByte, 1); \
} while (0)

/*******************************************************************************
 * @fn          main
 *
 * @brief       Runs the main routine
 *                
 * @param       none
 * * @return      none
 */
void main(void) {
  

  // Initialize MCU and peripherals
  initMCU();
  
  // Update LCD
  updateLcd();
  
  // Write radio registers
  registerConfig();
   
  // Calibrate radio according to errata
  manualCalibration(); 
  
  // Set up and enable interrupt on GPIO2 (PKT_SYNC_RXTX)
  trxIsrConnect(GPIO_2, RISING_EDGE, &syncReceivedISR);
  trxEnableInt(SYNC_RECEIVED);
  
  // Set up and enable interrupt on GPIO0 (RXFIFO_THR_PKT)
  trxIsrConnect(GPIO_0, RISING_EDGE, &fifoThresholdReachedISR);
  trxEnableInt(FIFO_THRESHOLD_REACHED); //Threshold Interrupt for both wM-Bus and DSSS packets
  
  // Set up and enable interrupt on GPIO3 (RXFIFO_THR_PKT)
  trxIsrConnect(GPIO_3, FALLING_EDGE, &packetReceivedISR);
  
  DUAL_SYNC_SETTINGS(); // search for wM-Bus and DSSS packets
  
  // Strobe RX to receive packet
  trxSpiCmdStrobe(CC112X_SRX);
   
  //Start Timer B as soon as the radio enters RX
  // The radio will stay in RX for 1 minute and in this time it should be
  // able to receive 1996 packets from the generator
  //----------------------------------------------------------------------------
#ifdef TIMEOUT
  do {
    cc112xSpiReadReg(CC112X_MARCSTATE, &readByte, 1);
  } while (readByte != 0x6D);
  
  TB0CTL = 0x0210;
  TB0CCTL0 = CCIE;
  TB0CCR0 = 16000;
#endif
  //----------------------------------------------------------------------------
   
  // Infinite loop
  while(TRUE) {
  
      // Initialize RX info variable 
    RXinfo.lengthField = 0;           // Length Field in the wireless MBUS packet
    RXinfo.length      = 0;           // Total length of bytes to receive packet
    RXinfo.bytesLeft   = 256;         // Bytes left to to be read from the RX FIFO
    RXinfo.pByteIndex  = rxBuffer;    // Pointer to current position in the byte array
    RXinfo.format      = INFINITE;    // Infinite or fixed packet mode
    RXinfo.start       = TRUE;        // Sync or End of Packet
    RXinfo.complete    = FALSE;       // Packet Received
    RXinfo.mode        = 1;           // wireless M-Bus mode is always T (includes C-mode)
    
    // wait for a sync word is found (dual sync used)
    while (!tMode && !dsssMode);
    
    if (tMode) {
      //------------------------------------------------------------------------
      // Begin T-Mode
      //------------------------------------------------------------------------          
        syncReceived = FALSE;
        
        while (!packetReceived);
  
        cc112xSpiReadRxFifo(RXinfo.pByteIndex, RXinfo.bytesLeft);  

        if (!cMode) {
             rxStatus = decodeRXBytesTmode(rxBuffer, RXbytes, packetSize(RXinfo.lengthField));
             
             if (rxStatus != 0) {
              if (rxStatus == 1)
                errorCounterTMode++;         
              else if (rxStatus == 2)
                errorCounterCRCTMode++;
              rxStatus = 0;
            }
            else  //No error found!
              packetCounterTMode++; //only count correctly decoded wM-Bus packets
             
        }
        else { //C-mode packet decoding (just CRC16 check and removal), Start at rxBuffer[2] ==> L-field 
             rxStatus = decodeTXPacket(RXbytes, &rxBuffer[2], packetSize(RXinfo.lengthField)); 
           if (rxStatus != 0) {
              if (rxStatus == 1)
                errorCounterCMode++;         
              else if (rxStatus == 2)
                errorCounterCRCCMode++;
              rxStatus = 0;
            }
            else  //No error found!
              packetCounterCMode++; //only count correctly decoded wM-Bus packets
          }
              
         trxSpiCmdStrobe(CC112X_SIDLE);
         trxSpiCmdStrobe(CC112X_SFRX);
   }  //------------------------------------------------------------------------
      // End T-Mode
      //------------------------------------------------------------------------
    
    
   else { 
        //------------------------------------------------------------------------
        // Begin DSSS Mode
        //------------------------------------------------------------------------
        timeout = 4000;//////////////////////////////////////////////////////////////////////////////////////////////////////
        while((!syncReceived) && ((--timeout) > 0));
        
        if (syncReceived) {
          //----------------------------------------------------------------------
          // Begin DSSS Sync is found
          //----------------------------------------------------------------------
          syncReceived = FALSE;
          
          // Re-configure for FIXED mode
      //    writeByte = FIXED_PACKET_LENGTH; cc112xSpiWriteReg(CC112X_PKT_CFG0, &writeByte, 1);

          writeByte = 196; cc112xSpiWriteReg(CC112X_PKT_LEN, &writeByte, 1);
 
          while (!thresholdReached);
          thresholdReached = FALSE;
      
          // Read 100 bytes from the RXFIFO
          cc112xSpiReadRxFifo(rxBuffer, 100);
      
          while (!packetReceived);
             
          // Read the rest of the packets from the RXFIFO
          cc112xSpiReadRxFifo(rxBuffer + 100, 96);
          
          recreatePacket();
      
          crcOK = checkCRC(original);
      
          if (!crcOK)
            errorCounterDSSSMode++;
          if (crcOK) 
            packetCounterDSSSMode++;
          
          //----------------------------------------------------------------------
          // End DSSS Sync is found
          //----------------------------------------------------------------------
        } 
        else {
          //----------------------------------------------------------------------
          // Begin Timeout Handling
          //----------------------------------------------------------------------
          trxSpiCmdStrobe(CC112X_SIDLE);
          do {
            cc112xSpiReadReg(CC112X_MARCSTATE, &readByte, 1);
          } while (readByte != 0x41);
        }
        
          //----------------------------------------------------------------------
          // End Timeout Handling
          //----------------------------------------------------------------------
        
        DUAL_SYNC_SETTINGS();        
        //------------------------------------------------------------------------
        // End DSSS Mode
        //------------------------------------------------------------------------
      }

    // Enter RX mode again
    trxDisableInt(PACKET_RECEIVED);
    
    //Re-configure for INFINITE length mode; RX FIFO THRSHD still 101
   // writeByte = INFINITE_PACKET_LENGTH; cc112xSpiWriteReg(CC112X_PKT_CFG0, &writeByte, 1);
    writeByte = 255; cc112xSpiWriteReg(CC112X_PKT_LEN, &writeByte, 1);
    writeByte = RX_FIFO_THRESHOLD; 
    cc112xSpiWriteReg(CC112X_FIFO_CFG, &writeByte, 1);
    RXinfo.start = TRUE;
    rxStatus = 0;
    
    trxClearIntFlag(SYNC_RECEIVED);
    trxEnableInt(SYNC_RECEIVED);
    
    trxClearIntFlag(FIFO_THRESHOLD_REACHED);
    trxEnableInt(FIFO_THRESHOLD_REACHED);
    
    packetReceived = FALSE;
    tMode = FALSE;
    cMode = FALSE;
    dsssMode = FALSE;
  
    trxSpiCmdStrobe(CC112X_SIDLE);
    trxSpiCmdStrobe(CC112X_SFRX);
    updateLcd();
    trxSpiCmdStrobe(CC112X_SRX);
    
  }
}


/*******************************************************************************
* @fn          syncReceivedISR
*
* @brief       
*
* @param       none
*
* @return      none
*/
static void syncReceivedISR(void) {
  
  uint8 writeByte;
  uint8 readByte;
  uint8 numRxBytes = 0;
  
  syncReceived = TRUE;
  
  if (!tMode && !dsssMode) {
    
    // This code will only run if in dual sync mode
    
    // Check to see which sync word is found
    cc112xSpiReadReg(CC112X_DEM_STATUS, &readByte, 1);
    
    if (readByte & 0x20) {
      //------------------------------------------------------------------------
      // T-Mode Sync Begin
      //------------------------------------------------------------------------
      tMode = TRUE;
      RXinfo.start = TRUE;
      thresholdReached = FALSE;
      trxDisableInt(SYNC_RECEIVED);
       
   // After the sync word is received one needs to wait for the three
    // length bytes to be put in the RX FIFO
    do {
        cc112xSpiReadReg(CC112X_NUM_RXBYTES, &numRxBytes, 1);
    } while (numRxBytes < 3);
    
    // Read the length byte and store it in the packetLength variable
    cc112xSpiReadRxFifo(rxBuffer, 3);
    RXinfo.pByteIndex += 3;
  
    // packetLength = T-mode decoding or in C-mode Byte#3 (Bytes#1 and 2 indicate C-mode and Frame Format) 
  
     if (rxBuffer[0] == 0x54) { //C-mode
           cMode = TRUE;
           if (rxBuffer[1] == 0x3D) {
                  frmFormatB = TRUE;
                  RXinfo.lengthField = rxBuffer[2]; //C-mode L-field (1st Byte after C-mode identification bytes
                  RXinfo.length = rxBuffer[2] + 1 + 2; //add L-field of 1 Byte
             //     writeByte = RXinfo.length; 
             //     cc112xSpiWriteReg(CC112X_PKT_LEN, &writeByte, 1); 
           }    
     }
     
     else { //T-mode decoding of first 3 bytes to obtain 2 Bytes L-field and C-field
       decode3outof6(&rxBuffer[0], bytesDecoded, 0);
       RXinfo.lengthField = RXbytes[0] = bytesDecoded[0];
       RXbytes[1]= bytesDecoded[1];
       readByte = packetSize(RXbytes[0]);
       RXinfo.length = byteSize(2 /*T-mode */, 0 /* rcv */ , readByte);          
     }
       //substract 3 (already read out) from the #over-the-air bytes 
     RXinfo.bytesLeft = RXinfo.length - 3; //Over-the-air Bytes are in rxBuffer[]
 
     //check for error on L-field
     if (RXinfo.lengthField < 15 || RXinfo.lengthField > 255 || RXinfo.length > MAX_FIXED_LENGTH)   {
       
       // Exit RX and flush RX FIFO due to Length-Field or Number of bytes (>256) being out of range  
       //using 256 Bytes Maximum Packet length of CC1120 Packet Engine 
        rxStatus = PACKET_LENGTH_ERROR;
        trxSpiCmdStrobe(CC112X_SIDLE);
        trxSpiCmdStrobe(CC112X_SFRX);
        RXinfo.start = FALSE;
        
        //set Flag to unblock PACKET RECEIVED Flag
        packetReceived = TRUE; 
     }
     
     
     else {   
       //Now reprogram the CC1125 FIFO Packet Engine with Length mode and #Bytes
       // Set fixed packet length mode if less than (256 Bytes)
       // Make sure that the packet length is in the correct range, update
        // variables and enable interrupt on falling edge of GPIO3 (PKT_SYNC_RXTX)
                
       if (RXinfo.length <= (MAX_FIXED_LENGTH ))   { //FIXED Length possible
         writeByte = FIXED_PACKET_LENGTH; cc112xSpiWriteReg(CC112X_PKT_CFG0, &writeByte, 1);               
          RXinfo.format = FIXED;

          if (RXinfo.length <= (128+3)) { //Rest of Packet will fit into RX FIFO completely 
             trxDisableInt(FIFO_THRESHOLD_REACHED);
             writeByte = RXinfo.length; 
   //          if (cMode) writeByte += 2; //add 2 Bytes for the C-mode SYNC and Frame Format type
             cc112xSpiWriteReg(CC112X_PKT_LEN, &writeByte, 1); 
             writeByte = RXinfo.length - 1;
             cc112xSpiWriteReg(CC112X_FIFO_CFG, &writeByte, 1);  //RX FIFO Threshold
          }
          else { //more than 128 Bytes left, set RX FIFO THRESHOLD to 101 bytes
            writeByte = RX_FIFO_THRESHOLD; 
            cc112xSpiWriteReg(CC112X_FIFO_CFG, &writeByte, 1);              
          }       
       }
          // Infinite packet length mode is more than 256 bytes          
       
       /*else  {
         writeByte = RXinfo.length  % (MAX_FIXED_LENGTH);
         cc112xSpiWriteReg(CC112X_PKT_LEN, &writeByte, 1);
          }   
       */
       
        }
    }
      
      //------------------------------------------------------------------------
      // T-Mode Sync End
      //------------------------------------------------------------------------
    
    else {
      //------------------------------------------------------------------------
      // DSSS Sync Begin
      //------------------------------------------------------------------------
      trxSpiCmdStrobe(CC112X_SIDLE);
      dsssMode = TRUE;
      
      // change settings to look for a 4 byte long sync word
      _4_BYTES_SYNC_SETTINGS();
      
     // trxEnableInt(SYNC_RECEIVED); 
      trxClearIntFlag(FIFO_THRESHOLD_REACHED);
      trxEnableInt(FIFO_THRESHOLD_REACHED); 
      
      syncReceived = FALSE;      
    
      trxSpiCmdStrobe(CC112X_SFRX);
      trxSpiCmdStrobe(CC112X_SRX); 
      
      //------------------------------------------------------------------------
      // DSSS Sync End
      //------------------------------------------------------------------------
    }
//    trxClearIntFlag(PACKET_RECEIVED);
//    trxEnableInt(PACKET_RECEIVED); 

  }
   // Clear ISR flag
  trxClearIntFlag(SYNC_RECEIVED);
  trxClearIntFlag(PACKET_RECEIVED);
  trxEnableInt(PACKET_RECEIVED);    //now we can also get Packet Interrupt before RX FIFO Threshold       

} 


/*******************************************************************************
* @fn         packetReceivedISR
*
* @brief      Function running when a packet has been received 
*             (T-mode or DSSS mode)
*
* @param      none
*
* @return     none
*/
static void packetReceivedISR(void) {
  packetReceived = TRUE;
  RXinfo.complete = TRUE;
  trxClearIntFlag(PACKET_RECEIVED);
  trxEnableInt(SYNC_RECEIVED);    //now we can also get Packet Interrupt before RX FIFO Threshold  
  trxEnableInt(FIFO_THRESHOLD_REACHED);
} 


/*******************************************************************************
* @fn         fifoThresholdReachedISR
*
* @brief      Function running when FIFO threshold is reached 
*             (101 Bytes are in the RXFIFO)
*
* @param      none
*
* @return     none
*/
static void fifoThresholdReachedISR(void) {
    
  thresholdReached = TRUE;
  if (dsssMode) {
    trxDisableInt(FIFO_THRESHOLD_REACHED);  //101 Bytes from DSSS packet were received
    //next interrupt should be PACKET_RECEIVED
  }
  else if (tMode || cMode) {
    // Change to fixed packet length mode when there is less than 256
    // bytes left to receive
  if ( ((RXinfo.bytesLeft - BYTES_IN_RX_FIFO) < MAX_FIXED_LENGTH )
        && (RXinfo.format == INFINITE)) {
        writeByte = FIXED_PACKET_LENGTH;
        cc112xSpiWriteReg(CC112X_PKT_CFG0, &writeByte, 1);
        RXinfo.format = FIXED;
  }

    // Read BYTES_IN_RX_FIFO bytes from the RX FIFO and update the variables
    // keeping track of how many more bytes need to be read and where in
    // rxBuffer they should be stored
    cc112xSpiReadRxFifo(RXinfo.pByteIndex, BYTES_IN_RX_FIFO);
    RXinfo.bytesLeft -= BYTES_IN_RX_FIFO;
    RXinfo.pByteIndex += BYTES_IN_RX_FIFO; 
  }
      
    // Clear ISR flag
  trxClearIntFlag(FIFO_THRESHOLD_REACHED);
}


/*******************************************************************************
* @fn         updateLcd
*
* @brief        
*
* @param        
*                 
* @return     none
*/
static void updateLcd(void) {
  lcdBufferClear(0);
   
  lcdBufferPrintString(0, "DSSS and T-/C-Mode RX",          0,  0);
  lcdBufferPrintString(0, "T-Md Packets:",          0,  1);
  lcdBufferPrintInt(0, (int32)packetCounterTMode,     90, 1);
  lcdBufferPrintString(0, "T-3of6 Errors:",           0,  2);
  lcdBufferPrintInt(0, (int32)errorCounterTMode,      90, 2);
  lcdBufferPrintString(0, "T- CRC Errors:",           0,  3); //Line 4
  lcdBufferPrintInt(0, (int32)errorCounterCRCTMode,      90, 3);     //Line 4
  lcdBufferPrintString(0, "DSSS Packets:",            0,  4);
  lcdBufferPrintInt(0, (int32)packetCounterDSSSMode,  90, 4);
  lcdBufferPrintString(0, "DSSS Errors:",             0,  5);
  lcdBufferPrintInt(0, (int32)errorCounterDSSSMode,   90, 5);
  lcdBufferPrintString(0, "C-Mode Packets:",          0,  6);
  lcdBufferPrintInt(0, (int32)packetCounterCMode,     90, 6);
  lcdBufferPrintString(0, "C-Mode Errors:",          0,  7);
  lcdBufferPrintInt(0, (int32)errorCounterCMode + errorCounterCRCCMode,     90, 7);  
  lcdSendBuffer(0);
}


/*******************************************************************************
* @fn           recreatePacket
*
* @brief        
*
* @param        
*                 
* @return      none
*/
static void recreatePacket(void) {
  uint8 k;
  uint16 temp;
  uint16 data0 = 0x00ED; // 11 bits barker code representing "0" 
  uint8 error = 0;        // Incremented for every bit in dataOut[i] that is 
                          // not equal to data0
  
  // Regroup the data in rxBuffer in chunks of 11: Take the data in the rxBuffer and group them into 11 bits
  partition(rxBuffer, 11, 196); //19.5 Bytes including CRC16: 195 Bytes and 2 Bits/chips are in the RX FIFO
  
  // Implement error correction on dataOut
  for (uint8 i = 0; i < 142; i++) {
    temp = dataOut[i];
    error = 0;
    for (uint8 j = 0; j < 11; j++) {
      if (((temp >> j) & 0x0001) ^ ((data0 >> j) & 0x0001))
        error++;
    }
    if (error > 5)
      dataOut[i] = 1;
    else
      dataOut[i] = 0;
  }
  
  // original[] will contain the complete DSSS packet 
  // (header, sync, payload, and CRC)
  original[0] = 0xAA;
  original[1] = 0xAD | (dataOut[0] << 1) | (dataOut[1] << 0); //add two last bits from 1st nibble = 0xD
  k = 2;
  for (uint8 i = 2; i < 19; i++) {
    original[i] = (uint8)(dataOut[k++] << 7);
    for (uint8 j = 7; j > 0; j--)
      original[i] |= (uint8)(dataOut[k++] << j -1);
  }
  original[19] = (dataOut[138] << 7) | (dataOut[139] << 6) | (dataOut[140] << 5) | (dataOut[141] << 4);
}


/*******************************************************************************
* @fn           partition
*
* @brief        Partition a byte string to n bits. Number of bytes in must be a 
*               multiple of the number of bits to partition with
*
* @param        dataIn
*                 Input byte string
*               n
*                 Number of bits per output word
*               nBytes
*                 Number of bytes in input string
*
* @return      none
*/
static void partition(uint8* dataIn, uint8 n, uint8 nBytes) {
  
  uint32 shifter = 0; 
  uint8 nBits = 0;    // Keeps track of the number of bits held by the shifter
  uint8 index = 0;    // Input data index
  uint16 tmp;
  uint8 j = 0;
    
  while (TRUE) {
        
    // If we have enough bits in the shifter, go get them
    if (nBits > (n - 1)) {
            
      // Mask out the bits we need
      dataOut[j++] = ((uint16)(shifter >> (nBits - n))) & 0x07FF;
           
      // Dump the used bits from the shifter
      nBits -= n;
          
      // If no more input bytes, we are done
      if (index == nBytes)
        break;
    } else {  // Not enough bits, get eight more
      tmp = dataIn[index];
      index += 1;
      shifter <<= 8; // 32 bits shift
      shifter |= tmp;
      nBits += 8;
    }
  }
}


/*******************************************************************************
* @fn          timerB0ISR
*
* @brief       Timer B generates an interrupt 60 s after RX mode was started.
*              The code does never return form this function
*
* @param       none
*
* @return      none
*/
#ifdef TIMEOUT
#pragma vector = TIMER0_B0_VECTOR
__interrupt void timerB0ISR(void) {
  static uint16 counter = 0;
  if (++counter == 60000) {
    trxSpiCmdStrobe(CC112X_SIDLE);
    while (TRUE);
  }
}
#endif


/******************************************************************************
 * @fn          initMCU
 *
 * @brief       Initialize MCU and board peripherals
 *                
 * @param       none
 *
 * @return      none
 */
static void initMCU(void) {
  
  // Stop watchdog timer to prevent time out reset
  WDTCTL = WDTPW + WDTHOLD;
  
  // Initialize all GPIO configurations
  halBoardInit();
  
  // Care must be taken when handling power modes
  // - Peripheral units can request clocks and have them granted even if
  //   the system is in a power mode. Peripheral clock request is enabled
  //   as default.
  // - Simple Link only needs ACLK to be enabled to timers
  //   during power mode operation
  halMcuDisablePeripheralClockRequest((MCLKREQEN+SMCLKREQEN));

  // SPI flash uses same SPI interface as LCD -- we'll disable the SPI flash
  P8SEL &= BIT6; // ioflash_csn = gp
  P8DIR |= BIT6; // tpflash_csn = out
  P8OUT |= BIT6; // flash_csn = 1  
  
  // Init leds 
  halLedInit();

  // Init Buttons
  halButtonsInit();
  halButtonsInterruptEnable();

  // Init LCD
  lcdInit();

  // Instantiate tranceiver RF spi interface 
  // input clockDivider - SMCLK/clockDivider gives SCLK frequency, 16MHz /4
  trxRfSpiInterfaceInit(0x02);//0x10
   
  halMcuSetSystemClock(HAL_MCU_SYSCLK_16MHZ);
  
  // Init UART to use with virtual COM port
  P5DIR |= 0x80;
  P5SEL |= 0xC0;
  
  UCA1CTL1 |= UCSWRST;
  
  UCA1CTL1 |= UCSSEL__SMCLK;   // SMCLK

  UCA1BR0 = 0x23;
  UCA1BR1 = 0x01;

  UCA1MCTL |= UCBRS_1 + UCBRF_0;

  UCA1CTL1 &= ~UCSWRST;

  // Enable global interrupt
  _BIS_SR(GIE);
}


/*******************************************************************************
* @fn          registerConfig
*
* @brief       Write register settings as given by SmartRF Studio
*
* @param       none
*
* @return      none
*/
static void registerConfig(void) {
  
  uint8 writeByte;
  
  // Reset radio
  trxSpiCmdStrobe(CC112X_SRES);
  
  // Write registers to radio
  for(uint16 i = 0; i < (sizeof  preferredSettings/sizeof(registerSetting_t)); i++) {
    writeByte =  preferredSettings[i].data;
    cc112xSpiWriteReg(preferredSettings[i].addr, &writeByte, 1);
  }
}


/******************************************************************************
 * @fn          manualCalibration
 *
 * @brief       calibrates radio according to CC112x errata
 *                
 * @param       none
 *
 * @return      none
 */
#define VCDAC_START_OFFSET 2
#define FS_VCO2_INDEX 0
#define FS_VCO4_INDEX 1
#define FS_CHP_INDEX 2
static void manualCalibration(void) {
  uint8 original_fs_cal2;
  uint8 calResults_for_vcdac_start_high[3];
  uint8 calResults_for_vcdac_start_mid[3];
  uint8 marcstate;
  uint8 writeByte;
    
  // 1) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
  writeByte = 0x00;
  cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    
  // 2) Start with high VCDAC (original VCDAC_START + 2):
  cc112xSpiReadReg(CC112X_FS_CAL2, &original_fs_cal2, 1);
  writeByte = original_fs_cal2 + VCDAC_START_OFFSET;
  cc112xSpiWriteReg(CC112X_FS_CAL2, &writeByte, 1);
    
  // 3) Calibrate and wait for calibration to be done (radio back in IDLE state)
  trxSpiCmdStrobe(CC112X_SCAL);
    
  do {
    cc112xSpiReadReg(CC112X_MARCSTATE, &marcstate, 1);
  } while (marcstate != 0x41);
    
  // 4) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with high VCDAC_START value
  cc112xSpiReadReg(CC112X_FS_VCO2, &calResults_for_vcdac_start_high[FS_VCO2_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_VCO4, &calResults_for_vcdac_start_high[FS_VCO4_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_CHP, &calResults_for_vcdac_start_high[FS_CHP_INDEX], 1);
    
  // 5) Set VCO cap-array to 0 (FS_VCO2 = 0x00)
  writeByte = 0x00;
  cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    
  // 6) Continue with mid VCDAC (original VCDAC_START):
  writeByte = original_fs_cal2;
  cc112xSpiWriteReg(CC112X_FS_CAL2, &writeByte, 1);
    
  // 7) Calibrate and wait for calibration to be done (radio back in IDLE state)
  trxSpiCmdStrobe(CC112X_SCAL);
    
  do {
    cc112xSpiReadReg(CC112X_MARCSTATE, &marcstate, 1);
  } while (marcstate != 0x41);
    
  // 8) Read FS_VCO2, FS_VCO4 and FS_CHP register obtained with mid VCDAC_START value
  cc112xSpiReadReg(CC112X_FS_VCO2, &calResults_for_vcdac_start_mid[FS_VCO2_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_VCO4, &calResults_for_vcdac_start_mid[FS_VCO4_INDEX], 1);
  cc112xSpiReadReg(CC112X_FS_CHP, &calResults_for_vcdac_start_mid[FS_CHP_INDEX], 1);
    
  // 9) Write back highest FS_VCO2 and corresponding FS_VCO and FS_CHP result
  if (calResults_for_vcdac_start_high[FS_VCO2_INDEX] > calResults_for_vcdac_start_mid[FS_VCO2_INDEX]) {
    writeByte = calResults_for_vcdac_start_high[FS_VCO2_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_high[FS_VCO4_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO4, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_high[FS_CHP_INDEX];
    cc112xSpiWriteReg(CC112X_FS_CHP, &writeByte, 1);
  } else {
    writeByte = calResults_for_vcdac_start_mid[FS_VCO2_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO2, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_mid[FS_VCO4_INDEX];
    cc112xSpiWriteReg(CC112X_FS_VCO4, &writeByte, 1);
    writeByte = calResults_for_vcdac_start_mid[FS_CHP_INDEX];
    cc112xSpiWriteReg(CC112X_FS_CHP, &writeByte, 1);
  }
}
/***********************************************************************************
  Copyright 2011 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
***********************************************************************************/

