/******************************************************************************
    Filename: crc.c  
    
    Description:  
                 
*******************************************************************************/


/******************************************************************************
 * DEFINES
 */
#define CRC16_POLYNOM        0x3D65


/******************************************************************************
 * INCLUDES
 */
#include "hal_types.h"
#include "hal_defs.h"
#include "crc.h"


/*******************************************************************************
* @fn          checkCRC
*
* @brief       Check the CRC of the incoming packet with 15 Bytes Payload + 2 Bytes CRC
*
* @param       receivedPacket: Pointer to received packet
*
* @return      TRUE: CRC OK
*              FALSE: CRC Not OK
*/
uint8 checkCRC(uint8 *receivedPacket) {
  
  uint16 checksumReceived;
  uint16 checksumCalc = 0;

  uint8 rxBufferCRC[17] = {0};
  
  for(uint8 i = 0; i < 17; i++)
    rxBufferCRC[i] = (receivedPacket[i + 2] << 4) | (receivedPacket[i + 3] >> 4);

  for(uint8 i = 0; i < 15; i++)  
    checksumCalc = calcCRC(rxBufferCRC[i], checksumCalc); 
    
  checksumCalc = ~checksumCalc;
 
  checksumReceived = (uint16)((uint16)(rxBufferCRC[15] << 8) | (uint16)rxBufferCRC[16]);
  
  if (checksumCalc == checksumReceived)
    return TRUE;
  else
    return FALSE;
}


/*******************************************************************************
* @fn          calcCRC
*
* @brief       Calculates the 16-bit CRC. The function requires that the 
*              CRC_POLYNOM is defined, which gives the wanted CRC polynom
*
* @param       crcData  - Data to perform the CRC-16 operation on.
*              crcReg   - Current or initial value of the CRC calculation
*
* @return      The value returned is the 16-bit CRC (of the data supplied so far)  
*/ 

uint16 calcCRC(uint8 crcData, uint16 crcReg) {
  uint8 i;

  for (i = 0; i < 8; i++) {
    if (((crcReg & 0x8000) >> 8) ^ (crcData & 0x80))
      crcReg = (crcReg << 1) ^ CRC16_POLYNOM;
    else
      crcReg = (crcReg << 1);

    crcData <<= 1;
  }
  return crcReg;
}
//-------------------------------------------------------------------------------------------------------
//  uint16 crcCalc(uint16 crcReg, uint8 crcData) 
//
//  DESCRIPTION:
//      Calculates the 16-bit CRC. The function requires that the CRC_POLYNOM is defined,
//      which gives the wanted CRC polynom. 
//
//  ARGUMENTS: 
//      uint8  crcData  - Data to perform the CRC-16 operation on.
//      uint16 crcReg   - Current or initial value of the CRC calculation
//
//  RETURN:
//      The value returned is the 16-bit CRC (of the data supplied so far). 
//-------------------------------------------------------------------------------------------------------

uint16 crcCalc(uint16 crcReg, uint8 crcData) 
{
  uint8 i;

 for (i = 0; i < 8; i++) 
  {
    // If upper most bit is 1
    if (((crcReg & 0x8000) >> 8) ^ (crcData & 0x80))
      crcReg = (crcReg << 1)  ^ CRC16_POLYNOM;
    else
      crcReg = (crcReg << 1);

    crcData <<= 1;
  }
  
  return crcReg;
}

/*  CRC_POLYNOM         0x3D65
The CRC shall be computed over the information from the previous block, and shall be generated according to FT3 of EN 60870-5-1. The formula is:
The CRC polynomial is: x16 + x13 + x12 + x11 + x10 + x8 + x6 + x5 + x2 + 1
The initial value is 0.
The final CRC is complemented.
*/

/******************************************************************************
  Copyright 2011 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
*******************************************************************************/
